@@ -1,3 +1,4 @@
+//go:build acceptance || blockstorage
 // +build acceptance blockstorage
 
 package extensions
@@ -13,12 +14,8 @@ import (
 
 func TestSchedulerStatsList(t *testing.T) {
 	clients.RequireAdmin(t)
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-	clients.SkipRelease(t, "stable/ocata")
-	clients.SkipRelease(t, "stable/pike")
 
-	blockClient, err := clients.NewBlockStorageV2Client()
+	blockClient, err := clients.NewBlockStorageV3Client()
 	th.AssertNoErr(t, err)
 
 	listOpts := schedulerstats.ListOpts{
