@@ -1,3 +1,4 @@
+//go:build acceptance || blockstorage
 // +build acceptance blockstorage
 
 package v1
@@ -11,6 +12,7 @@ import (
 )
 
 func TestVolumeTypesList(t *testing.T) {
+	clients.SkipReleasesAbove(t, "stable/icehouse")
 	client, err := clients.NewBlockStorageV1Client()
 	if err != nil {
 		t.Fatalf("Unable to create a blockstorage client: %v", err)
@@ -32,6 +34,7 @@ func TestVolumeTypesList(t *testing.T) {
 }
 
 func TestVolumeTypesCreateDestroy(t *testing.T) {
+	clients.SkipReleasesAbove(t, "stable/icehouse")
 	client, err := clients.NewBlockStorageV1Client()
 	if err != nil {
 		t.Fatalf("Unable to create a blockstorage client: %v", err)
