@@ -1,3 +1,4 @@
+//go:build acceptance || blockstorage
 // +build acceptance blockstorage
 
 package v3
@@ -6,13 +7,13 @@ import (
 	"testing"
 
 	"github.com/gophercloud/gophercloud/acceptance/clients"
+	"github.com/gophercloud/gophercloud/acceptance/tools"
 	"github.com/gophercloud/gophercloud/openstack/blockstorage/v3/snapshots"
 	"github.com/gophercloud/gophercloud/pagination"
 	th "github.com/gophercloud/gophercloud/testhelper"
 )
 
 func TestSnapshots(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
 	clients.RequireLong(t)
 
 	client, err := clients.NewBlockStorageV3Client()
@@ -26,6 +27,21 @@ func TestSnapshots(t *testing.T) {
 	th.AssertNoErr(t, err)
 	defer DeleteSnapshot(t, client, snapshot1)
 
+	// Update snapshot
+	updatedSnapshotName := tools.RandomString("ACPTTEST", 16)
+	updatedSnapshotDescription := tools.RandomString("ACPTTEST", 16)
+	updateOpts := snapshots.UpdateOpts{
+		Name:        &updatedSnapshotName,
+		Description: &updatedSnapshotDescription,
+	}
+	t.Logf("Attempting to update snapshot: %s", updatedSnapshotName)
+	updatedSnapshot, err := snapshots.Update(client, snapshot1.ID, updateOpts).Extract()
+	th.AssertNoErr(t, err)
+
+	tools.PrintResource(t, updatedSnapshot)
+	th.AssertEquals(t, updatedSnapshot.Name, updatedSnapshotName)
+	th.AssertEquals(t, updatedSnapshot.Description, updatedSnapshotDescription)
+
 	volume2, err := CreateVolume(t, client)
 	th.AssertNoErr(t, err)
 	defer DeleteVolume(t, client, volume2)
