@@ -151,7 +151,7 @@ func CreateBootableVolumeServer(t *testing.T, client *gophercloud.ServiceClient,
 		Name:      name,
 		FlavorRef: choices.FlavorID,
 		Networks: []servers.Network{
-			servers.Network{UUID: networkID},
+			{UUID: networkID},
 		},
 	}
 
@@ -208,15 +208,20 @@ func CreateDefaultRule(t *testing.T, client *gophercloud.ServiceClient) (dsr.Def
 // An error will be returned if the flavor could not be created.
 func CreateFlavor(t *testing.T, client *gophercloud.ServiceClient) (*flavors.Flavor, error) {
 	flavorName := tools.RandomString("flavor_", 5)
+	flavorDescription := fmt.Sprintf("I am %s and i am a yummy flavor", flavorName)
+
+	// Microversion 2.55 is required to add description to flavor
+	client.Microversion = "2.55"
 	t.Logf("Attempting to create flavor %s", flavorName)
 
 	isPublic := true
 	createOpts := flavors.CreateOpts{
-		Name:     flavorName,
-		RAM:      1,
-		VCPUs:    1,
-		Disk:     gophercloud.IntToPointer(1),
-		IsPublic: &isPublic,
+		Name:        flavorName,
+		RAM:         1,
+		VCPUs:       1,
+		Disk:        gophercloud.IntToPointer(1),
+		IsPublic:    &isPublic,
+		Description: flavorDescription,
 	}
 
 	flavor, err := flavors.Create(client, createOpts).Extract()
@@ -231,6 +236,7 @@ func CreateFlavor(t *testing.T, client *gophercloud.ServiceClient) (*flavors.Fla
 	th.AssertEquals(t, flavor.Disk, 1)
 	th.AssertEquals(t, flavor.VCPUs, 1)
 	th.AssertEquals(t, flavor.IsPublic, true)
+	th.AssertEquals(t, flavor.Description, flavorDescription)
 
 	return flavor, nil
 }
@@ -320,7 +326,7 @@ func CreateMultiEphemeralServer(t *testing.T, client *gophercloud.ServiceClient,
 		FlavorRef: choices.FlavorID,
 		ImageRef:  choices.ImageID,
 		Networks: []servers.Network{
-			servers.Network{UUID: networkID},
+			{UUID: networkID},
 		},
 	}
 
@@ -456,7 +462,7 @@ func CreateServer(t *testing.T, client *gophercloud.ServiceClient) (*servers.Ser
 		ImageRef:  choices.ImageID,
 		AdminPass: pwd,
 		Networks: []servers.Network{
-			servers.Network{UUID: networkID},
+			{UUID: networkID},
 		},
 		Metadata: map[string]string{
 			"abc": "def",
@@ -516,7 +522,7 @@ func CreateMicroversionServer(t *testing.T, client *gophercloud.ServiceClient) (
 		ImageRef:  choices.ImageID,
 		AdminPass: pwd,
 		Networks: []servers.Network{
-			servers.Network{UUID: networkID},
+			{UUID: networkID},
 		},
 		Metadata: map[string]string{
 			"abc": "def",
@@ -567,7 +573,7 @@ func CreateServerWithoutImageRef(t *testing.T, client *gophercloud.ServiceClient
 		FlavorRef: choices.FlavorID,
 		AdminPass: pwd,
 		Networks: []servers.Network{
-			servers.Network{UUID: networkID},
+			{UUID: networkID},
 		},
 		Personality: servers.Personality{
 			&servers.File{
@@ -610,7 +616,7 @@ func CreateServerWithTags(t *testing.T, client *gophercloud.ServiceClient, netwo
 		ImageRef:  choices.ImageID,
 		AdminPass: pwd,
 		Networks: []servers.Network{
-			servers.Network{UUID: networkID},
+			{UUID: networkID},
 		},
 		Metadata: map[string]string{
 			"abc": "def",
@@ -719,7 +725,7 @@ func CreateServerInServerGroup(t *testing.T, client *gophercloud.ServiceClient,
 		ImageRef:  choices.ImageID,
 		AdminPass: pwd,
 		Networks: []servers.Network{
-			servers.Network{UUID: networkID},
+			{UUID: networkID},
 		},
 	}
 
@@ -771,7 +777,7 @@ func CreateServerWithPublicKey(t *testing.T, client *gophercloud.ServiceClient,
 		FlavorRef: choices.FlavorID,
 		ImageRef:  choices.ImageID,
 		Networks: []servers.Network{
-			servers.Network{UUID: networkID},
+			{UUID: networkID},
 		},
 	}
 
@@ -802,8 +808,13 @@ func CreateServerWithPublicKey(t *testing.T, client *gophercloud.ServiceClient,
 // CreateVolumeAttachment will attach a volume to a server. An error will be
 // returned if the volume failed to attach.
 func CreateVolumeAttachment(t *testing.T, client *gophercloud.ServiceClient, blockClient *gophercloud.ServiceClient, server *servers.Server, volume *volumes.Volume) (*volumeattach.VolumeAttachment, error) {
+	tag := tools.RandomString("ACPTTEST", 16)
+	dot := false
+
 	volumeAttachOptions := volumeattach.CreateOpts{
-		VolumeID: volume.ID,
+		VolumeID:            volume.ID,
+		Tag:                 tag,
+		DeleteOnTermination: dot,
 	}
 
 	t.Logf("Attempting to attach volume %s to server %s", volume.ID, server.ID)
@@ -870,7 +881,7 @@ func DeleteFloatingIP(t *testing.T, client *gophercloud.ServiceClient, floatingI
 // the keypair failed to be deleted. This works best when used as a deferred
 // function.
 func DeleteKeyPair(t *testing.T, client *gophercloud.ServiceClient, keyPair *keypairs.KeyPair) {
-	err := keypairs.Delete(client, keyPair.Name).ExtractErr()
+	err := keypairs.Delete(client, keyPair.Name, nil).ExtractErr()
 	if err != nil {
 		t.Fatalf("Unable to delete keypair %s: %v", keyPair.Name, err)
 	}
@@ -1135,7 +1146,7 @@ func WaitForComputeStatus(client *gophercloud.ServiceClient, server *servers.Ser
 	})
 }
 
-//Convenience method to fill an QuotaSet-UpdateOpts-struct from a QuotaSet-struct
+// Convenience method to fill an QuotaSet-UpdateOpts-struct from a QuotaSet-struct
 func FillUpdateOptsFromQuotaSet(src quotasets.QuotaSet, dest *quotasets.UpdateOpts) {
 	dest.FixedIPs = &src.FixedIPs
 	dest.FloatingIPs = &src.FloatingIPs
