@@ -1,3 +1,4 @@
+//go:build acceptance || compute || servers
 // +build acceptance compute servers
 
 package v2
@@ -11,8 +12,6 @@ import (
 )
 
 func TestMigrate(t *testing.T) {
-	t.Skip("This is not passing in OpenLab. Works locally")
-
 	clients.RequireLong(t)
 	clients.RequireAdmin(t)
 
