@@ -1,3 +1,4 @@
+//go:build acceptance || compute || servers
 // +build acceptance compute servers
 
 package v2
@@ -86,8 +87,6 @@ func TestServersCreateDestroy(t *testing.T) {
 }
 
 func TestServersWithExtensionsCreateDestroy(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-
 	clients.RequireLong(t)
 
 	var extendedServer struct {
@@ -490,9 +489,6 @@ func TestServersConsoleOutput(t *testing.T) {
 
 func TestServersTags(t *testing.T) {
 	clients.RequireLong(t)
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-	clients.SkipRelease(t, "stable/ocata")
 
 	choices, err := clients.AcceptanceTestChoicesFromEnv()
 	th.AssertNoErr(t, err)
@@ -566,9 +562,6 @@ func TestServersTags(t *testing.T) {
 func TestServersWithExtendedAttributesCreateDestroy(t *testing.T) {
 	clients.RequireLong(t)
 	clients.RequireAdmin(t)
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-	clients.SkipRelease(t, "stable/ocata")
 
 	client, err := clients.NewComputeV2Client()
 	th.AssertNoErr(t, err)
