@@ -1,3 +1,4 @@
+//go:build acceptance
 // +build acceptance
 
 package v3
@@ -57,8 +58,7 @@ func TestRolesCRUD(t *testing.T) {
 	th.AssertNoErr(t, err)
 
 	createOpts := roles.CreateOpts{
-		Name:     "testrole",
-		DomainID: "default",
+		Name: "testrole",
 		Extra: map[string]interface{}{
 			"description": "test role description",
 		},
@@ -72,9 +72,7 @@ func TestRolesCRUD(t *testing.T) {
 	tools.PrintResource(t, role)
 	tools.PrintResource(t, role.Extra)
 
-	listOpts := roles.ListOpts{
-		DomainID: "default",
-	}
+	listOpts := roles.ListOpts{}
 	allPages, err := roles.List(client, listOpts).AllPages()
 	th.AssertNoErr(t, err)
 
@@ -111,23 +109,11 @@ func TestRolesCRUD(t *testing.T) {
 func TestRolesFilterList(t *testing.T) {
 	clients.RequireAdmin(t)
 
-	// For some reason this is not longer working.
-	// It might be a temporary issue.
-	clients.SkipRelease(t, "master")
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-	clients.SkipRelease(t, "stable/ocata")
-	clients.SkipRelease(t, "stable/pike")
-	clients.SkipRelease(t, "stable/queens")
-	clients.SkipRelease(t, "stable/rocky")
-	clients.SkipRelease(t, "stable/stein")
-
 	client, err := clients.NewIdentityV3Client()
 	th.AssertNoErr(t, err)
 
 	createOpts := roles.CreateOpts{
-		Name:     "testrole",
-		DomainID: "default",
+		Name: "testrole",
 		Extra: map[string]interface{}{
 			"description": "test role description",
 		},
@@ -140,7 +126,7 @@ func TestRolesFilterList(t *testing.T) {
 
 	var listOpts roles.ListOpts
 	listOpts.Filters = map[string]string{
-		"name__contains": "TEST",
+		"name__contains": "test",
 	}
 
 	allPages, err := roles.List(client, listOpts).AllPages()
@@ -184,6 +170,74 @@ func TestRolesFilterList(t *testing.T) {
 	th.AssertEquals(t, found, false)
 }
 
+func TestRoleListAssignmentIncludeNamesAndSubtree(t *testing.T) {
+	clients.RequireAdmin(t)
+
+	client, err := clients.NewIdentityV3Client()
+	th.AssertNoErr(t, err)
+
+	project, err := CreateProject(t, client, nil)
+	th.AssertNoErr(t, err)
+	defer DeleteProject(t, client, project.ID)
+
+	domainID := "default"
+	roleCreateOpts := roles.CreateOpts{
+		DomainID: domainID,
+	}
+	role, err := CreateRole(t, client, &roleCreateOpts)
+	th.AssertNoErr(t, err)
+	defer DeleteRole(t, client, role.ID)
+
+	user, err := CreateUser(t, client, nil)
+	th.AssertNoErr(t, err)
+	defer DeleteUser(t, client, user.ID)
+
+	t.Logf("Attempting to assign a role %s to a user %s on a project %s",
+		role.Name, user.Name, project.Name)
+
+	assignOpts := roles.AssignOpts{
+		UserID:    user.ID,
+		ProjectID: project.ID,
+	}
+	err = roles.Assign(client, role.ID, assignOpts).ExtractErr()
+	th.AssertNoErr(t, err)
+
+	t.Logf("Successfully assigned a role %s to a user %s on a project %s",
+		role.Name, user.Name, project.Name)
+
+	defer UnassignRole(t, client, role.ID, &roles.UnassignOpts{
+		UserID:    user.ID,
+		ProjectID: project.ID,
+	})
+
+	iTrue := true
+	listAssignmentsOpts := roles.ListAssignmentsOpts{
+		UserID:         user.ID,
+		ScopeProjectID: domainID, // set domainID in ScopeProjectID field to list assignments on all projects in domain
+		IncludeSubtree: &iTrue,
+		IncludeNames:   &iTrue,
+	}
+	allPages, err := roles.ListAssignments(client, listAssignmentsOpts).AllPages()
+	th.AssertNoErr(t, err)
+
+	allRoles, err := roles.ExtractRoleAssignments(allPages)
+	th.AssertNoErr(t, err)
+
+	t.Logf("Role assignments(with names) of user %s on projects in domain %s:", user.Name, domainID)
+	var found bool
+	for _, _role := range allRoles {
+		tools.PrintResource(t, _role)
+		if _role.Role.ID == role.ID &&
+			_role.User.Name == user.Name &&
+			_role.Scope.Project.Name == project.Name &&
+			_role.Scope.Project.Domain.ID == domainID {
+			found = true
+		}
+	}
+
+	th.AssertEquals(t, found, true)
+}
+
 func TestRoleListAssignmentForUserOnProject(t *testing.T) {
 	clients.RequireAdmin(t)
 
@@ -483,10 +537,12 @@ func TestRolesAssignToUserOnProject(t *testing.T) {
 		ProjectID: project.ID,
 	})
 
+	iTrue := true
 	lao := roles.ListAssignmentsOpts{
 		RoleID:         role.ID,
 		ScopeProjectID: project.ID,
 		UserID:         user.ID,
+		IncludeNames:   &iTrue,
 	}
 
 	allPages, err := roles.ListAssignments(client, lao).AllPages()
@@ -503,6 +559,10 @@ func TestRolesAssignToUserOnProject(t *testing.T) {
 		if roleAssignment.Role.ID == role.ID {
 			found = true
 		}
+
+		if roleAssignment.User.Domain.ID == "" || roleAssignment.Scope.Project.Domain.ID == "" {
+			found = false
+		}
 	}
 
 	th.AssertEquals(t, found, true)
@@ -550,10 +610,12 @@ func TestRolesAssignToUserOnDomain(t *testing.T) {
 		DomainID: domain.ID,
 	})
 
+	iTrue := true
 	lao := roles.ListAssignmentsOpts{
 		RoleID:        role.ID,
 		ScopeDomainID: domain.ID,
 		UserID:        user.ID,
+		IncludeNames:  &iTrue,
 	}
 
 	allPages, err := roles.ListAssignments(client, lao).AllPages()
@@ -570,6 +632,10 @@ func TestRolesAssignToUserOnDomain(t *testing.T) {
 		if roleAssignment.Role.ID == role.ID {
 			found = true
 		}
+
+		if roleAssignment.User.Domain.ID == "" {
+			found = false
+		}
 	}
 
 	th.AssertEquals(t, found, true)
@@ -620,10 +686,12 @@ func TestRolesAssignToGroupOnDomain(t *testing.T) {
 		DomainID: domain.ID,
 	})
 
+	iTrue := true
 	lao := roles.ListAssignmentsOpts{
 		RoleID:        role.ID,
 		ScopeDomainID: domain.ID,
 		GroupID:       group.ID,
+		IncludeNames:  &iTrue,
 	}
 
 	allPages, err := roles.ListAssignments(client, lao).AllPages()
@@ -640,6 +708,10 @@ func TestRolesAssignToGroupOnDomain(t *testing.T) {
 		if roleAssignment.Role.ID == role.ID {
 			found = true
 		}
+
+		if roleAssignment.Group.Domain.ID == "" {
+			found = false
+		}
 	}
 
 	th.AssertEquals(t, found, true)
@@ -687,10 +759,12 @@ func TestRolesAssignToGroupOnProject(t *testing.T) {
 		ProjectID: project.ID,
 	})
 
+	iTrue := true
 	lao := roles.ListAssignmentsOpts{
 		RoleID:         role.ID,
 		ScopeProjectID: project.ID,
 		GroupID:        group.ID,
+		IncludeNames:   &iTrue,
 	}
 
 	allPages, err := roles.ListAssignments(client, lao).AllPages()
@@ -707,6 +781,10 @@ func TestRolesAssignToGroupOnProject(t *testing.T) {
 		if roleAssignment.Role.ID == role.ID {
 			found = true
 		}
+
+		if roleAssignment.Scope.Project.Domain.ID == "" || roleAssignment.Group.Domain.ID == "" {
+			found = false
+		}
 	}
 
 	th.AssertEquals(t, found, true)
