@@ -1,3 +1,4 @@
+//go:build acceptance || networking || loadbalancer || loadbalancers
 // +build acceptance networking loadbalancer loadbalancers
 
 package v2
@@ -7,6 +8,7 @@ import (
 
 	"github.com/gophercloud/gophercloud/acceptance/clients"
 	networking "github.com/gophercloud/gophercloud/acceptance/openstack/networking/v2"
+	"github.com/gophercloud/gophercloud/acceptance/openstack/networking/v2/extensions/qos/policies"
 	"github.com/gophercloud/gophercloud/acceptance/tools"
 	"github.com/gophercloud/gophercloud/openstack/loadbalancer/v2/l7policies"
 	"github.com/gophercloud/gophercloud/openstack/loadbalancer/v2/listeners"
@@ -17,13 +19,6 @@ import (
 )
 
 func TestLoadbalancersList(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-	clients.SkipRelease(t, "stable/ocata")
-	clients.SkipRelease(t, "stable/pike")
-	clients.SkipRelease(t, "stable/queens")
-	clients.SkipRelease(t, "stable/rocky")
-
 	client, err := clients.NewLoadBalancerV2Client()
 	th.AssertNoErr(t, err)
 
@@ -39,13 +34,6 @@ func TestLoadbalancersList(t *testing.T) {
 }
 
 func TestLoadbalancersListByTags(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-	clients.SkipRelease(t, "stable/ocata")
-	clients.SkipRelease(t, "stable/pike")
-	clients.SkipRelease(t, "stable/queens")
-	clients.SkipRelease(t, "stable/rocky")
-
 	netClient, err := clients.NewNetworkV2Client()
 	th.AssertNoErr(t, err)
 
@@ -63,7 +51,7 @@ func TestLoadbalancersListByTags(t *testing.T) {
 	// Add "test" tag intentionally to test the "not-tags" parameter. Because "test" tag is also used in other test
 	// cases, we use "test" tag to exclude load balancers created by other test case.
 	tags := []string{"tag1", "tag2", "test"}
-	lb, err := CreateLoadBalancer(t, lbClient, subnet.ID, tags)
+	lb, err := CreateLoadBalancer(t, lbClient, subnet.ID, tags, "")
 	th.AssertNoErr(t, err)
 	defer DeleteLoadBalancer(t, lbClient, lb.ID)
 
@@ -108,14 +96,7 @@ func TestLoadbalancersListByTags(t *testing.T) {
 	th.AssertEquals(t, 0, len(allLoadbalancers))
 }
 
-func TestLoadbalancersCRUD(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-	clients.SkipRelease(t, "stable/ocata")
-	clients.SkipRelease(t, "stable/pike")
-	clients.SkipRelease(t, "stable/queens")
-	clients.SkipRelease(t, "stable/rocky")
-
+func TestLoadbalancerHTTPCRUD(t *testing.T) {
 	netClient, err := clients.NewNetworkV2Client()
 	th.AssertNoErr(t, err)
 
@@ -130,73 +111,15 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	th.AssertNoErr(t, err)
 	defer networking.DeleteSubnet(t, netClient, subnet.ID)
 
-	tags := []string{"test"}
-	lb, err := CreateLoadBalancer(t, lbClient, subnet.ID, tags)
+	lb, err := CreateLoadBalancer(t, lbClient, subnet.ID, nil, "")
 	th.AssertNoErr(t, err)
 	defer DeleteLoadBalancer(t, lbClient, lb.ID)
 
-	lbDescription := ""
-	updateLoadBalancerOpts := loadbalancers.UpdateOpts{
-		Description: &lbDescription,
-	}
-	_, err = loadbalancers.Update(lbClient, lb.ID, updateLoadBalancerOpts).Extract()
-	th.AssertNoErr(t, err)
-
-	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
-		t.Fatalf("Timed out waiting for loadbalancer to become active")
-	}
-
-	newLB, err := loadbalancers.Get(lbClient, lb.ID).Extract()
-	th.AssertNoErr(t, err)
-
-	tools.PrintResource(t, newLB)
-
-	th.AssertEquals(t, newLB.Description, lbDescription)
-
-	lbStats, err := loadbalancers.GetStats(lbClient, lb.ID).Extract()
-	th.AssertNoErr(t, err)
-
-	tools.PrintResource(t, lbStats)
-
-	// Because of the time it takes to create a loadbalancer,
-	// this test will include some other resources.
-
 	// Listener
-	listener, err := CreateListener(t, lbClient, lb)
+	listener, err := CreateListenerHTTP(t, lbClient, lb)
 	th.AssertNoErr(t, err)
 	defer DeleteListener(t, lbClient, lb.ID, listener.ID)
 
-	listenerName := ""
-	listenerDescription := ""
-	listenerHeaders := map[string]string{
-		"X-Forwarded-For": "true",
-	}
-	updateListenerOpts := listeners.UpdateOpts{
-		Name:          &listenerName,
-		Description:   &listenerDescription,
-		InsertHeaders: &listenerHeaders,
-	}
-	_, err = listeners.Update(lbClient, listener.ID, updateListenerOpts).Extract()
-	th.AssertNoErr(t, err)
-
-	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
-		t.Fatalf("Timed out waiting for loadbalancer to become active")
-	}
-
-	newListener, err := listeners.Get(lbClient, listener.ID).Extract()
-	th.AssertNoErr(t, err)
-
-	tools.PrintResource(t, newListener)
-
-	th.AssertEquals(t, newListener.Name, listenerName)
-	th.AssertEquals(t, newListener.Description, listenerDescription)
-	th.AssertDeepEquals(t, newListener.InsertHeaders, listenerHeaders)
-
-	listenerStats, err := listeners.GetStats(lbClient, listener.ID).Extract()
-	th.AssertNoErr(t, err)
-
-	tools.PrintResource(t, listenerStats)
-
 	// L7 policy
 	policy, err := CreateL7Policy(t, lbClient, listener, lb)
 	th.AssertNoErr(t, err)
@@ -209,7 +132,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	_, err = l7policies.Update(lbClient, policy.ID, updateL7policyOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -241,7 +164,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	_, err = l7policies.UpdateRule(lbClient, policy.ID, rule.ID, updateL7ruleOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -251,7 +174,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	tools.PrintResource(t, newRule)
 
 	// Pool
-	pool, err := CreatePool(t, lbClient, lb)
+	pool, err := CreatePoolHTTP(t, lbClient, lb)
 	th.AssertNoErr(t, err)
 	defer DeletePool(t, lbClient, lb.ID, pool.ID)
 
@@ -264,7 +187,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	_, err = pools.Update(lbClient, pool.ID, updatePoolOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -285,7 +208,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	_, err = l7policies.Update(lbClient, policy.ID, updateL7policyOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -303,16 +226,119 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	defer DeleteL7Policy(t, lbClient, lb.ID, policy.ID)
 	defer DeleteL7Rule(t, lbClient, lb.ID, policy.ID, rule.ID)
 
-	// Update listener's default pool ID and remove headers
-	listenerHeaders = map[string]string{}
-	updateListenerOpts = listeners.UpdateOpts{
-		DefaultPoolID: &pool.ID,
-		InsertHeaders: &listenerHeaders,
+	// Member
+	member, err := CreateMember(t, lbClient, lb, pool, subnet.ID, subnet.CIDR)
+	th.AssertNoErr(t, err)
+	defer DeleteMember(t, lbClient, lb.ID, pool.ID, member.ID)
+
+	monitor, err := CreateMonitor(t, lbClient, lb, pool)
+	th.AssertNoErr(t, err)
+	defer DeleteMonitor(t, lbClient, lb.ID, monitor.ID)
+}
+
+func TestLoadbalancersCRUD(t *testing.T) {
+	netClient, err := clients.NewNetworkV2Client()
+	th.AssertNoErr(t, err)
+
+	// Create QoS policy first as the loadbalancer and its port
+	//needs to be deleted before the QoS policy can be deleted
+	policy2, err := policies.CreateQoSPolicy(t, netClient)
+	th.AssertNoErr(t, err)
+	defer policies.DeleteQoSPolicy(t, netClient, policy2.ID)
+
+	lbClient, err := clients.NewLoadBalancerV2Client()
+	th.AssertNoErr(t, err)
+
+	network, err := networking.CreateNetwork(t, netClient)
+	th.AssertNoErr(t, err)
+	defer networking.DeleteNetwork(t, netClient, network.ID)
+
+	subnet, err := networking.CreateSubnet(t, netClient, network.ID)
+	th.AssertNoErr(t, err)
+	defer networking.DeleteSubnet(t, netClient, subnet.ID)
+
+	policy1, err := policies.CreateQoSPolicy(t, netClient)
+	th.AssertNoErr(t, err)
+	defer policies.DeleteQoSPolicy(t, netClient, policy1.ID)
+
+	tags := []string{"test"}
+	lb, err := CreateLoadBalancer(t, lbClient, subnet.ID, tags, policy1.ID)
+	th.AssertNoErr(t, err)
+	th.AssertEquals(t, lb.VipQosPolicyID, policy1.ID)
+	defer DeleteLoadBalancer(t, lbClient, lb.ID)
+
+	lbDescription := ""
+	updateLoadBalancerOpts := loadbalancers.UpdateOpts{
+		Description:    &lbDescription,
+		VipQosPolicyID: &policy2.ID,
+	}
+	_, err = loadbalancers.Update(lbClient, lb.ID, updateLoadBalancerOpts).Extract()
+	th.AssertNoErr(t, err)
+
+	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
+		t.Fatalf("Timed out waiting for loadbalancer to become active")
+	}
+
+	newLB, err := loadbalancers.Get(lbClient, lb.ID).Extract()
+	th.AssertNoErr(t, err)
+
+	tools.PrintResource(t, newLB)
+
+	th.AssertEquals(t, newLB.Description, lbDescription)
+	th.AssertEquals(t, newLB.VipQosPolicyID, policy2.ID)
+
+	lbStats, err := loadbalancers.GetStats(lbClient, lb.ID).Extract()
+	th.AssertNoErr(t, err)
+
+	tools.PrintResource(t, lbStats)
+
+	// Because of the time it takes to create a loadbalancer,
+	// this test will include some other resources.
+
+	// Listener
+	listener, err := CreateListener(t, lbClient, lb)
+	th.AssertNoErr(t, err)
+	defer DeleteListener(t, lbClient, lb.ID, listener.ID)
+
+	listenerName := ""
+	listenerDescription := ""
+	updateListenerOpts := listeners.UpdateOpts{
+		Name:        &listenerName,
+		Description: &listenerDescription,
 	}
 	_, err = listeners.Update(lbClient, listener.ID, updateListenerOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
+		t.Fatalf("Timed out waiting for loadbalancer to become active")
+	}
+
+	newListener, err := listeners.Get(lbClient, listener.ID).Extract()
+	th.AssertNoErr(t, err)
+
+	tools.PrintResource(t, newListener)
+
+	th.AssertEquals(t, newListener.Name, listenerName)
+	th.AssertEquals(t, newListener.Description, listenerDescription)
+
+	listenerStats, err := listeners.GetStats(lbClient, listener.ID).Extract()
+	th.AssertNoErr(t, err)
+
+	tools.PrintResource(t, listenerStats)
+
+	// Pool
+	pool, err := CreatePool(t, lbClient, lb)
+	th.AssertNoErr(t, err)
+	defer DeletePool(t, lbClient, lb.ID, pool.ID)
+
+	// Update listener's default pool ID.
+	updateListenerOpts = listeners.UpdateOpts{
+		DefaultPoolID: &pool.ID,
+	}
+	_, err = listeners.Update(lbClient, listener.ID, updateListenerOpts).Extract()
+	th.AssertNoErr(t, err)
+
+	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -322,7 +348,6 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	tools.PrintResource(t, newListener)
 
 	th.AssertEquals(t, newListener.DefaultPoolID, pool.ID)
-	th.AssertDeepEquals(t, newListener.InsertHeaders, listenerHeaders)
 
 	// Remove listener's default pool ID
 	emptyPoolID := ""
@@ -332,7 +357,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	_, err = listeners.Update(lbClient, listener.ID, updateListenerOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -344,7 +369,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	th.AssertEquals(t, newListener.DefaultPoolID, "")
 
 	// Member
-	member, err := CreateMember(t, lbClient, lb, newPool, subnet.ID, subnet.CIDR)
+	member, err := CreateMember(t, lbClient, lb, pool, subnet.ID, subnet.CIDR)
 	th.AssertNoErr(t, err)
 	defer DeleteMember(t, lbClient, lb.ID, pool.ID, member.ID)
 
@@ -357,7 +382,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	_, err = pools.UpdateMember(lbClient, pool.ID, member.ID, updateMemberOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -378,7 +403,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 		t.Fatalf("Unable to batch update members")
 	}
 
-	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -387,22 +412,13 @@ func TestLoadbalancersCRUD(t *testing.T) {
 
 	tools.PrintResource(t, newMember)
 
-	// delete members from the pool
-	if err = pools.BatchUpdateMembers(lbClient, pool.ID, []pools.BatchUpdateMemberOpts{}).ExtractErr(); err != nil {
-		t.Fatalf("Unable to delete members")
-	}
-
-	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
-		t.Fatalf("Timed out waiting for loadbalancer to become active")
-	}
-
 	pool, err = pools.Get(lbClient, pool.ID).Extract()
 	th.AssertNoErr(t, err)
 
 	tools.PrintResource(t, pool)
 
 	// Monitor
-	monitor, err := CreateMonitor(t, lbClient, lb, newPool)
+	monitor, err := CreateMonitor(t, lbClient, lb, pool)
 	th.AssertNoErr(t, err)
 	defer DeleteMonitor(t, lbClient, lb.ID, monitor.ID)
 
@@ -417,7 +433,7 @@ func TestLoadbalancersCRUD(t *testing.T) {
 	_, err = monitors.Update(lbClient, monitor.ID, updateMonitorOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err = WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -432,13 +448,6 @@ func TestLoadbalancersCRUD(t *testing.T) {
 }
 
 func TestLoadbalancersCascadeCRUD(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-	clients.SkipRelease(t, "stable/ocata")
-	clients.SkipRelease(t, "stable/pike")
-	clients.SkipRelease(t, "stable/queens")
-	clients.SkipRelease(t, "stable/rocky")
-
 	netClient, err := clients.NewNetworkV2Client()
 	th.AssertNoErr(t, err)
 
@@ -454,7 +463,7 @@ func TestLoadbalancersCascadeCRUD(t *testing.T) {
 	defer networking.DeleteSubnet(t, netClient, subnet.ID)
 
 	tags := []string{"test"}
-	lb, err := CreateLoadBalancer(t, lbClient, subnet.ID, tags)
+	lb, err := CreateLoadBalancer(t, lbClient, subnet.ID, tags, "")
 	th.AssertNoErr(t, err)
 	defer CascadeDeleteLoadBalancer(t, lbClient, lb.ID)
 
@@ -477,7 +486,7 @@ func TestLoadbalancersCascadeCRUD(t *testing.T) {
 	_, err = listeners.Update(lbClient, listener.ID, updateListenerOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -497,7 +506,7 @@ func TestLoadbalancersCascadeCRUD(t *testing.T) {
 	_, err = pools.Update(lbClient, pool.ID, updatePoolOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -517,7 +526,7 @@ func TestLoadbalancersCascadeCRUD(t *testing.T) {
 	_, err = pools.UpdateMember(lbClient, pool.ID, member.ID, updateMemberOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -539,7 +548,7 @@ func TestLoadbalancersCascadeCRUD(t *testing.T) {
 	_, err = monitors.Update(lbClient, monitor.ID, updateMonitorOpts).Extract()
 	th.AssertNoErr(t, err)
 
-	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE", loadbalancerActiveTimeoutSeconds); err != nil {
+	if err := WaitForLoadBalancerState(lbClient, lb.ID, "ACTIVE"); err != nil {
 		t.Fatalf("Timed out waiting for loadbalancer to become active")
 	}
 
@@ -551,3 +560,29 @@ func TestLoadbalancersCascadeCRUD(t *testing.T) {
 	th.AssertEquals(t, newMonitor.Delay, newDelay)
 	th.AssertEquals(t, newMonitor.MaxRetriesDown, newMaxRetriesDown)
 }
+
+func TestLoadbalancersFullyPopulatedCRUD(t *testing.T) {
+	netClient, err := clients.NewNetworkV2Client()
+	th.AssertNoErr(t, err)
+
+	lbClient, err := clients.NewLoadBalancerV2Client()
+	th.AssertNoErr(t, err)
+
+	network, err := networking.CreateNetwork(t, netClient)
+	th.AssertNoErr(t, err)
+	defer networking.DeleteNetwork(t, netClient, network.ID)
+
+	subnet, err := networking.CreateSubnet(t, netClient, network.ID)
+	th.AssertNoErr(t, err)
+	defer networking.DeleteSubnet(t, netClient, subnet.ID)
+
+	tags := []string{"test"}
+	lb, err := CreateLoadBalancerFullyPopulated(t, lbClient, subnet.ID, tags)
+	th.AssertNoErr(t, err)
+	defer CascadeDeleteLoadBalancer(t, lbClient, lb.ID)
+
+	newLB, err := loadbalancers.Get(lbClient, lb.ID).Extract()
+	th.AssertNoErr(t, err)
+
+	tools.PrintResource(t, newLB)
+}
