@@ -1,3 +1,4 @@
+//go:build acceptance || networking || loadbalancer || pools
 // +build acceptance networking loadbalancer pools
 
 package v2
@@ -11,13 +12,6 @@ import (
 )
 
 func TestPoolsList(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-	clients.SkipRelease(t, "stable/ocata")
-	clients.SkipRelease(t, "stable/pike")
-	clients.SkipRelease(t, "stable/queens")
-	clients.SkipRelease(t, "stable/rocky")
-
 	client, err := clients.NewLoadBalancerV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a loadbalancer client: %v", err)
