@@ -88,7 +88,7 @@ func CreatePort(t *testing.T, client *gophercloud.ServiceClient, networkID, subn
 		Name:         portName,
 		Description:  portDescription,
 		AdminStateUp: gophercloud.Enabled,
-		FixedIPs:     []ports.IP{ports.IP{SubnetID: subnetID}},
+		FixedIPs:     []ports.IP{{SubnetID: subnetID}},
 	}
 
 	port, err := ports.Create(client, createOpts).Extract()
@@ -96,7 +96,7 @@ func CreatePort(t *testing.T, client *gophercloud.ServiceClient, networkID, subn
 		return port, err
 	}
 
-	if err := WaitForPortToCreate(client, port.ID, 60); err != nil {
+	if err := WaitForPortToCreate(client, port.ID); err != nil {
 		return port, err
 	}
 
@@ -125,7 +125,7 @@ func CreatePortWithNoSecurityGroup(t *testing.T, client *gophercloud.ServiceClie
 		NetworkID:      networkID,
 		Name:           portName,
 		AdminStateUp:   &iFalse,
-		FixedIPs:       []ports.IP{ports.IP{SubnetID: subnetID}},
+		FixedIPs:       []ports.IP{{SubnetID: subnetID}},
 		SecurityGroups: &[]string{},
 	}
 
@@ -134,7 +134,7 @@ func CreatePortWithNoSecurityGroup(t *testing.T, client *gophercloud.ServiceClie
 		return port, err
 	}
 
-	if err := WaitForPortToCreate(client, port.ID, 60); err != nil {
+	if err := WaitForPortToCreate(client, port.ID); err != nil {
 		return port, err
 	}
 
@@ -161,7 +161,7 @@ func CreatePortWithoutPortSecurity(t *testing.T, client *gophercloud.ServiceClie
 		NetworkID:    networkID,
 		Name:         portName,
 		AdminStateUp: gophercloud.Enabled,
-		FixedIPs:     []ports.IP{ports.IP{SubnetID: subnetID}},
+		FixedIPs:     []ports.IP{{SubnetID: subnetID}},
 	}
 
 	iFalse := false
@@ -175,7 +175,7 @@ func CreatePortWithoutPortSecurity(t *testing.T, client *gophercloud.ServiceClie
 		return port, err
 	}
 
-	if err := WaitForPortToCreate(client, port.ID, 60); err != nil {
+	if err := WaitForPortToCreate(client, port.ID); err != nil {
 		return port, err
 	}
 
@@ -202,7 +202,7 @@ func CreatePortWithExtraDHCPOpts(t *testing.T, client *gophercloud.ServiceClient
 		NetworkID:    networkID,
 		Name:         portName,
 		AdminStateUp: gophercloud.Enabled,
-		FixedIPs:     []ports.IP{ports.IP{SubnetID: subnetID}},
+		FixedIPs:     []ports.IP{{SubnetID: subnetID}},
 	}
 
 	createOpts := extradhcpopts.CreateOptsExt{
@@ -221,7 +221,7 @@ func CreatePortWithExtraDHCPOpts(t *testing.T, client *gophercloud.ServiceClient
 		return nil, err
 	}
 
-	if err := WaitForPortToCreate(client, port.ID, 60); err != nil {
+	if err := WaitForPortToCreate(client, port.ID); err != nil {
 		return nil, err
 	}
 
@@ -248,7 +248,7 @@ func CreatePortWithMultipleFixedIPs(t *testing.T, client *gophercloud.ServiceCli
 		Name:         portName,
 		Description:  portDescription,
 		AdminStateUp: gophercloud.Enabled,
-		FixedIPs:     []ports.IP{ports.IP{SubnetID: subnetID}, ports.IP{SubnetID: subnetID}},
+		FixedIPs:     []ports.IP{{SubnetID: subnetID}, {SubnetID: subnetID}},
 	}
 
 	port, err := ports.Create(client, createOpts).Extract()
@@ -256,7 +256,7 @@ func CreatePortWithMultipleFixedIPs(t *testing.T, client *gophercloud.ServiceCli
 		return port, err
 	}
 
-	if err := WaitForPortToCreate(client, port.ID, 60); err != nil {
+	if err := WaitForPortToCreate(client, port.ID); err != nil {
 		return port, err
 	}
 
@@ -312,6 +312,45 @@ func CreateSubnet(t *testing.T, client *gophercloud.ServiceClient, networkID str
 	return subnet, nil
 }
 
+// CreateSubnet will create a subnet on the specified Network ID and service types.
+//
+//	An error will be returned if the subnet could not be created.
+func CreateSubnetWithServiceTypes(t *testing.T, client *gophercloud.ServiceClient, networkID string) (*subnets.Subnet, error) {
+	subnetName := tools.RandomString("TESTACC-", 8)
+	subnetDescription := tools.RandomString("TESTACC-DESC-", 8)
+	subnetOctet := tools.RandomInt(1, 250)
+	subnetCIDR := fmt.Sprintf("192.168.%d.0/24", subnetOctet)
+	subnetGateway := fmt.Sprintf("192.168.%d.1", subnetOctet)
+	serviceTypes := []string{"network:routed"}
+	createOpts := subnets.CreateOpts{
+		NetworkID:    networkID,
+		CIDR:         subnetCIDR,
+		IPVersion:    4,
+		Name:         subnetName,
+		Description:  subnetDescription,
+		EnableDHCP:   gophercloud.Disabled,
+		GatewayIP:    &subnetGateway,
+		ServiceTypes: serviceTypes,
+	}
+
+	t.Logf("Attempting to create subnet: %s", subnetName)
+
+	subnet, err := subnets.Create(client, createOpts).Extract()
+	if err != nil {
+		return subnet, err
+	}
+
+	t.Logf("Successfully created subnet.")
+
+	th.AssertEquals(t, subnet.Name, subnetName)
+	th.AssertEquals(t, subnet.Description, subnetDescription)
+	th.AssertEquals(t, subnet.GatewayIP, subnetGateway)
+	th.AssertEquals(t, subnet.CIDR, subnetCIDR)
+	th.AssertDeepEquals(t, subnet.ServiceTypes, serviceTypes)
+
+	return subnet, nil
+}
+
 // CreateSubnetWithDefaultGateway will create a subnet on the specified Network
 // ID and have Neutron set the gateway by default An error will be returned if
 // the subnet could not be created.
@@ -514,8 +553,8 @@ func DeleteSubnet(t *testing.T, client *gophercloud.ServiceClient, subnetID stri
 	t.Logf("Deleted subnet: %s", subnetID)
 }
 
-func WaitForPortToCreate(client *gophercloud.ServiceClient, portID string, secs int) error {
-	return gophercloud.WaitFor(secs, func() (bool, error) {
+func WaitForPortToCreate(client *gophercloud.ServiceClient, portID string) error {
+	return tools.WaitFor(func() (bool, error) {
 		p, err := ports.Get(client, portID).Extract()
 		if err != nil {
 			return false, err
