@@ -1,3 +1,4 @@
+//go:build acceptance
 // +build acceptance
 
 package v1
@@ -11,8 +12,6 @@ import (
 )
 
 func TestStackEvents(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-
 	client, err := clients.NewOrchestrationV1Client()
 	th.AssertNoErr(t, err)
 
