@@ -1,3 +1,6 @@
+//go:build acceptance
+// +build acceptance
+
 package v2
 
 import (
@@ -10,9 +13,6 @@ import (
 )
 
 func TestShareCreate(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -33,9 +33,6 @@ func TestShareCreate(t *testing.T) {
 }
 
 func TestShareExportLocations(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -48,7 +45,7 @@ func TestShareExportLocations(t *testing.T) {
 
 	defer DeleteShare(t, client, share)
 
-	err = waitForStatus(t, client, share.ID, "available", 120)
+	_, err = waitForStatus(t, client, share.ID, "available")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
@@ -70,9 +67,6 @@ func TestShareExportLocations(t *testing.T) {
 }
 
 func TestShareUpdate(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create shared file system client: %v", err)
@@ -122,9 +116,6 @@ func TestShareUpdate(t *testing.T) {
 }
 
 func TestShareListDetail(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -148,10 +139,6 @@ func TestShareListDetail(t *testing.T) {
 }
 
 func TestGrantAndRevokeAccess(t *testing.T) {
-	t.Skip("Currently failing in OpenLab")
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -178,10 +165,6 @@ func TestGrantAndRevokeAccess(t *testing.T) {
 }
 
 func TestListAccessRights(t *testing.T) {
-	t.Skip("Currently failing in OpenLab")
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -217,9 +200,6 @@ func TestListAccessRights(t *testing.T) {
 }
 
 func TestExtendAndShrink(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -239,7 +219,7 @@ func TestExtendAndShrink(t *testing.T) {
 	}
 
 	// We need to wait till the Extend operation is done
-	err = waitForStatus(t, client, share.ID, "available", 120)
+	_, err = waitForStatus(t, client, share.ID, "available")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
@@ -253,7 +233,7 @@ func TestExtendAndShrink(t *testing.T) {
 	}
 
 	// We need to wait till the Shrink operation is done
-	err = waitForStatus(t, client, share.ID, "available", 300)
+	_, err = waitForStatus(t, client, share.ID, "available")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
@@ -263,9 +243,6 @@ func TestExtendAndShrink(t *testing.T) {
 }
 
 func TestShareMetadata(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -325,9 +302,6 @@ func TestShareMetadata(t *testing.T) {
 }
 
 func TestRevert(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -341,7 +315,7 @@ func TestRevert(t *testing.T) {
 
 	defer DeleteShare(t, client, share)
 
-	err = waitForStatus(t, client, share.ID, "available", 120)
+	_, err = waitForStatus(t, client, share.ID, "available")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
@@ -350,10 +324,9 @@ func TestRevert(t *testing.T) {
 	if err != nil {
 		t.Fatalf("Unable to create a snapshot: %v", err)
 	}
-
 	defer DeleteSnapshot(t, client, snapshot)
 
-	err = waitForSnapshotStatus(t, client, snapshot.ID, "available", 120)
+	err = waitForSnapshotStatus(t, client, snapshot.ID, "available")
 	if err != nil {
 		t.Fatalf("Snapshot status error: %v", err)
 	}
@@ -367,18 +340,87 @@ func TestRevert(t *testing.T) {
 	}
 
 	// We need to wait till the Extend operation is done
-	err = waitForStatus(t, client, share.ID, "available", 120)
+	_, err = waitForStatus(t, client, share.ID, "available")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
 
+	err = waitForSnapshotStatus(t, client, snapshot.ID, "available")
+	if err != nil {
+		t.Fatalf("Snapshot status error: %v", err)
+	}
+
 	t.Logf("Share %s successfuly reverted", share.ID)
 }
 
-func TestResetStatus(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
+func TestShareRestoreFromSnapshot(t *testing.T) {
+	client, err := clients.NewSharedFileSystemV2Client()
+	if err != nil {
+		t.Fatalf("Unable to create a shared file system client: %v", err)
+	}
+	client.Microversion = "2.27"
 
+	shareType := "default"
+	share, err := CreateShare(t, client, shareType)
+	if err != nil {
+		t.Fatalf("Unable to create a share: %v", err)
+	}
+
+	defer DeleteShare(t, client, share)
+
+	_, err = waitForStatus(t, client, share.ID, "available")
+	if err != nil {
+		t.Fatalf("Share status error: %v", err)
+	}
+
+	snapshot, err := CreateSnapshot(t, client, share.ID)
+	if err != nil {
+		t.Fatalf("Unable to create a snapshot: %v", err)
+	}
+	defer DeleteSnapshot(t, client, snapshot)
+
+	err = waitForSnapshotStatus(t, client, snapshot.ID, "available")
+	if err != nil {
+		t.Fatalf("Snapshot status error: %v", err)
+	}
+
+	// create a bigger share from a snapshot
+	iTrue := true
+	newSize := share.Size + 1
+	createOpts := shares.CreateOpts{
+		Size:        newSize,
+		Name:        "My Test Share",
+		Description: "My Test Description",
+		ShareProto:  "NFS",
+		ShareType:   shareType,
+		SnapshotID:  snapshot.ID,
+		IsPublic:    &iTrue,
+	}
+	restored, err := shares.Create(client, createOpts).Extract()
+	if err != nil {
+		t.Fatalf("Unable to create a share from a snapshot: %v", err)
+	}
+	defer DeleteShare(t, client, restored)
+
+	if restored.Size != newSize {
+		t.Fatalf("Unexpected restored share size: %d", restored.Size)
+	}
+
+	// We need to wait till the Extend operation is done
+	checkShare, err := waitForStatus(t, client, restored.ID, "available")
+	if err != nil {
+		t.Fatalf("Share status error: %v", err)
+	}
+
+	t.Logf("Share %s has been successfully restored: %+#v", checkShare.ID, checkShare)
+
+	err = waitForSnapshotStatus(t, client, snapshot.ID, "available")
+	if err != nil {
+		t.Fatalf("Snapshot status error: %v", err)
+	}
+}
+
+func TestResetStatus(t *testing.T) {
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -392,7 +434,7 @@ func TestResetStatus(t *testing.T) {
 
 	defer DeleteShare(t, client, share)
 
-	err = waitForStatus(t, client, share.ID, "available", 120)
+	_, err = waitForStatus(t, client, share.ID, "available")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
@@ -406,7 +448,7 @@ func TestResetStatus(t *testing.T) {
 	}
 
 	// We need to wait till the Extend operation is done
-	err = waitForStatus(t, client, share.ID, "error", 120)
+	_, err = waitForStatus(t, client, share.ID, "error")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
@@ -415,9 +457,6 @@ func TestResetStatus(t *testing.T) {
 }
 
 func TestForceDelete(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
-
 	client, err := clients.NewSharedFileSystemV2Client()
 	if err != nil {
 		t.Fatalf("Unable to create a shared file system client: %v", err)
@@ -431,7 +470,7 @@ func TestForceDelete(t *testing.T) {
 
 	defer DeleteShare(t, client, share)
 
-	err = waitForStatus(t, client, share.ID, "available", 120)
+	_, err = waitForStatus(t, client, share.ID, "available")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
@@ -441,7 +480,7 @@ func TestForceDelete(t *testing.T) {
 		t.Fatalf("Unable to force delete a share: %v", err)
 	}
 
-	err = waitForStatus(t, client, share.ID, "deleted", 120)
+	_, err = waitForStatus(t, client, share.ID, "deleted")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
@@ -450,8 +489,6 @@ func TestForceDelete(t *testing.T) {
 }
 
 func TestUnmanage(t *testing.T) {
-	clients.SkipRelease(t, "stable/mitaka")
-	clients.SkipRelease(t, "stable/newton")
 	clients.RequireAdmin(t)
 
 	client, err := clients.NewSharedFileSystemV2Client()
@@ -467,7 +504,7 @@ func TestUnmanage(t *testing.T) {
 
 	defer DeleteShare(t, client, share)
 
-	err = waitForStatus(t, client, share.ID, "available", 120)
+	_, err = waitForStatus(t, client, share.ID, "available")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
@@ -477,7 +514,7 @@ func TestUnmanage(t *testing.T) {
 		t.Fatalf("Unable to unmanage a share: %v", err)
 	}
 
-	err = waitForStatus(t, client, share.ID, "deleted", 120)
+	_, err = waitForStatus(t, client, share.ID, "deleted")
 	if err != nil {
 		t.Fatalf("Share status error: %v", err)
 	}
