@@ -5,10 +5,12 @@ import (
 	"crypto/hmac"
 	"crypto/md5"
 	"crypto/sha1"
+	"crypto/sha256"
+	"crypto/sha512"
 	"fmt"
+	"hash"
 	"io"
 	"io/ioutil"
-	"net/url"
 	"strings"
 	"time"
 
@@ -18,6 +20,25 @@ import (
 	"github.com/gophercloud/gophercloud/pagination"
 )
 
+// ErrTempURLKeyNotFound is an error indicating that the Temp URL key was
+// neigther set nor resolved from a container or account metadata.
+type ErrTempURLKeyNotFound struct{ gophercloud.ErrMissingInput }
+
+func (e ErrTempURLKeyNotFound) Error() string {
+	return "Unable to obtain the Temp URL key."
+}
+
+// ErrTempURLDigestNotValid is an error indicating that the requested
+// cryptographic hash function is not supported.
+type ErrTempURLDigestNotValid struct {
+	gophercloud.ErrMissingInput
+	Digest string
+}
+
+func (e ErrTempURLDigestNotValid) Error() string {
+	return fmt.Sprintf("The requested %q digest is not supported.", e.Digest)
+}
+
 // ListOptsBuilder allows extensions to add additional parameters to the List
 // request.
 type ListOptsBuilder interface {
@@ -38,6 +59,7 @@ type ListOpts struct {
 	Prefix    string `q:"prefix"`
 	Delimiter string `q:"delimiter"`
 	Path      string `q:"path"`
+	Versions  bool   `q:"versions"`
 }
 
 // ToObjectListParams formats a ListOpts into a query string and boolean
@@ -52,9 +74,12 @@ func (opts ListOpts) ToObjectListParams() (bool, string, error) {
 // pass the ListResult response to the ExtractInfo or ExtractNames function,
 // respectively.
 func List(c *gophercloud.ServiceClient, containerName string, opts ListOptsBuilder) pagination.Pager {
-	headers := map[string]string{"Accept": "text/plain", "Content-Type": "text/plain"}
+	url, err := listURL(c, containerName)
+	if err != nil {
+		return pagination.Pager{Err: err}
+	}
 
-	url := listURL(c, url.QueryEscape(containerName))
+	headers := map[string]string{"Accept": "text/plain", "Content-Type": "text/plain"}
 	if opts != nil {
 		full, query, err := opts.ToObjectListParams()
 		if err != nil {
@@ -93,6 +118,7 @@ type DownloadOpts struct {
 	Expires           string    `q:"expires"`
 	MultipartManifest string    `q:"multipart-manifest"`
 	Signature         string    `q:"signature"`
+	ObjectVersionID   string    `q:"version-id"`
 }
 
 // ToObjectDownloadParams formats a DownloadOpts into a query string and map of
@@ -106,14 +132,25 @@ func (opts DownloadOpts) ToObjectDownloadParams() (map[string]string, string, er
 	if err != nil {
 		return nil, q.String(), err
 	}
+	if !opts.IfModifiedSince.IsZero() {
+		h["If-Modified-Since"] = opts.IfModifiedSince.Format(time.RFC1123)
+	}
+	if !opts.IfUnmodifiedSince.IsZero() {
+		h["If-Unmodified-Since"] = opts.IfUnmodifiedSince.Format(time.RFC1123)
+	}
 	return h, q.String(), nil
 }
 
 // Download is a function that retrieves the content and metadata for an object.
-// To extract just the content, pass the DownloadResult response to the
-// ExtractContent function.
+// To extract just the content, call the DownloadResult method ExtractContent,
+// after checking DownloadResult's Err field.
 func Download(c *gophercloud.ServiceClient, containerName, objectName string, opts DownloadOptsBuilder) (r DownloadResult) {
-	url := downloadURL(c, url.QueryEscape(containerName), url.QueryEscape(objectName))
+	url, err := downloadURL(c, containerName, objectName)
+	if err != nil {
+		r.Err = err
+		return
+	}
+
 	h := make(map[string]string)
 	if opts != nil {
 		headers, query, err := opts.ToObjectDownloadParams()
@@ -153,8 +190,8 @@ type CreateOpts struct {
 	ContentLength      int64  `h:"Content-Length"`
 	ContentType        string `h:"Content-Type"`
 	CopyFrom           string `h:"X-Copy-From"`
-	DeleteAfter        int    `h:"X-Delete-After"`
-	DeleteAt           int    `h:"X-Delete-At"`
+	DeleteAfter        int64  `h:"X-Delete-After"`
+	DeleteAt           int64  `h:"X-Delete-At"`
 	DetectContentType  string `h:"X-Detect-Content-Type"`
 	ETag               string `h:"ETag"`
 	IfNoneMatch        string `h:"If-None-Match"`
@@ -219,7 +256,11 @@ func (opts CreateOpts) ToObjectCreateParams() (io.Reader, map[string]string, str
 // checksum, the failed request will automatically be retried up to a maximum
 // of 3 times.
 func Create(c *gophercloud.ServiceClient, containerName, objectName string, opts CreateOptsBuilder) (r CreateResult) {
-	url := createURL(c, url.QueryEscape(containerName), url.QueryEscape(objectName))
+	url, err := createURL(c, containerName, objectName)
+	if err != nil {
+		r.Err = err
+		return
+	}
 	h := make(map[string]string)
 	var b io.Reader
 	if opts != nil {
@@ -248,6 +289,12 @@ type CopyOptsBuilder interface {
 	ToObjectCopyMap() (map[string]string, error)
 }
 
+// CopyOptsQueryBuilder allows extensions to add additional query parameters to
+// the Copy request.
+type CopyOptsQueryBuilder interface {
+	ToObjectCopyQuery() (string, error)
+}
+
 // CopyOpts is a structure that holds parameters for copying one object to
 // another.
 type CopyOpts struct {
@@ -256,6 +303,7 @@ type CopyOpts struct {
 	ContentEncoding    string `h:"Content-Encoding"`
 	ContentType        string `h:"Content-Type"`
 	Destination        string `h:"Destination" required:"true"`
+	ObjectVersionID    string `q:"version-id"`
 }
 
 // ToObjectCopyMap formats a CopyOpts into a map of headers.
@@ -270,20 +318,42 @@ func (opts CopyOpts) ToObjectCopyMap() (map[string]string, error) {
 	return h, nil
 }
 
+// ToObjectCopyQuery formats a CopyOpts into a query.
+func (opts CopyOpts) ToObjectCopyQuery() (string, error) {
+	q, err := gophercloud.BuildQueryString(opts)
+	if err != nil {
+		return "", err
+	}
+	return q.String(), nil
+}
+
 // Copy is a function that copies one object to another.
 func Copy(c *gophercloud.ServiceClient, containerName, objectName string, opts CopyOptsBuilder) (r CopyResult) {
+	url, err := copyURL(c, containerName, objectName)
+	if err != nil {
+		r.Err = err
+		return
+	}
+
 	h := make(map[string]string)
 	headers, err := opts.ToObjectCopyMap()
 	if err != nil {
 		r.Err = err
 		return
 	}
-
 	for k, v := range headers {
 		h[k] = v
 	}
 
-	url := copyURL(c, url.QueryEscape(containerName), url.QueryEscape(objectName))
+	if opts, ok := opts.(CopyOptsQueryBuilder); ok {
+		query, err := opts.ToObjectCopyQuery()
+		if err != nil {
+			r.Err = err
+			return
+		}
+		url += query
+	}
+
 	resp, err := c.Request("COPY", url, &gophercloud.RequestOpts{
 		MoreHeaders: h,
 		OkCodes:     []int{201},
@@ -301,6 +371,7 @@ type DeleteOptsBuilder interface {
 // DeleteOpts is a structure that holds parameters for deleting an object.
 type DeleteOpts struct {
 	MultipartManifest string `q:"multipart-manifest"`
+	ObjectVersionID   string `q:"version-id"`
 }
 
 // ToObjectDeleteQuery formats a DeleteOpts into a query string.
@@ -311,7 +382,11 @@ func (opts DeleteOpts) ToObjectDeleteQuery() (string, error) {
 
 // Delete is a function that deletes an object.
 func Delete(c *gophercloud.ServiceClient, containerName, objectName string, opts DeleteOptsBuilder) (r DeleteResult) {
-	url := deleteURL(c, url.QueryEscape(containerName), url.QueryEscape(objectName))
+	url, err := deleteURL(c, containerName, objectName)
+	if err != nil {
+		r.Err = err
+		return
+	}
 	if opts != nil {
 		query, err := opts.ToObjectDeleteQuery()
 		if err != nil {
@@ -334,9 +409,10 @@ type GetOptsBuilder interface {
 // GetOpts is a structure that holds parameters for getting an object's
 // metadata.
 type GetOpts struct {
-	Newest    bool   `h:"X-Newest"`
-	Expires   string `q:"expires"`
-	Signature string `q:"signature"`
+	Newest          bool   `h:"X-Newest"`
+	Expires         string `q:"expires"`
+	Signature       string `q:"signature"`
+	ObjectVersionID string `q:"version-id"`
 }
 
 // ToObjectGetParams formats a GetOpts into a query string and a map of headers.
@@ -356,7 +432,11 @@ func (opts GetOpts) ToObjectGetParams() (map[string]string, string, error) {
 // the custom metadata, pass the GetResult response to the ExtractMetadata
 // function.
 func Get(c *gophercloud.ServiceClient, containerName, objectName string, opts GetOptsBuilder) (r GetResult) {
-	url := getURL(c, url.QueryEscape(containerName), url.QueryEscape(objectName))
+	url, err := getURL(c, containerName, objectName)
+	if err != nil {
+		r.Err = err
+		return
+	}
 	h := make(map[string]string)
 	if opts != nil {
 		headers, query, err := opts.ToObjectGetParams()
@@ -388,12 +468,13 @@ type UpdateOptsBuilder interface {
 // deleting an object's metadata.
 type UpdateOpts struct {
 	Metadata           map[string]string
-	ContentDisposition string `h:"Content-Disposition"`
-	ContentEncoding    string `h:"Content-Encoding"`
-	ContentType        string `h:"Content-Type"`
-	DeleteAfter        int    `h:"X-Delete-After"`
-	DeleteAt           int    `h:"X-Delete-At"`
-	DetectContentType  bool   `h:"X-Detect-Content-Type"`
+	RemoveMetadata     []string
+	ContentDisposition *string `h:"Content-Disposition"`
+	ContentEncoding    *string `h:"Content-Encoding"`
+	ContentType        *string `h:"Content-Type"`
+	DeleteAfter        *int64  `h:"X-Delete-After"`
+	DeleteAt           *int64  `h:"X-Delete-At"`
+	DetectContentType  *bool   `h:"X-Detect-Content-Type"`
 }
 
 // ToObjectUpdateMap formats a UpdateOpts into a map of headers.
@@ -402,14 +483,24 @@ func (opts UpdateOpts) ToObjectUpdateMap() (map[string]string, error) {
 	if err != nil {
 		return nil, err
 	}
+
 	for k, v := range opts.Metadata {
 		h["X-Object-Meta-"+k] = v
 	}
+
+	for _, k := range opts.RemoveMetadata {
+		h["X-Remove-Object-Meta-"+k] = "remove"
+	}
 	return h, nil
 }
 
 // Update is a function that creates, updates, or deletes an object's metadata.
 func Update(c *gophercloud.ServiceClient, containerName, objectName string, opts UpdateOptsBuilder) (r UpdateResult) {
+	url, err := updateURL(c, containerName, objectName)
+	if err != nil {
+		r.Err = err
+		return
+	}
 	h := make(map[string]string)
 	if opts != nil {
 		headers, err := opts.ToObjectUpdateMap()
@@ -422,7 +513,6 @@ func Update(c *gophercloud.ServiceClient, containerName, objectName string, opts
 			h[k] = v
 		}
 	}
-	url := updateURL(c, url.QueryEscape(containerName), url.QueryEscape(objectName))
 	resp, err := c.Post(url, nil, nil, &gophercloud.RequestOpts{
 		MoreHeaders: h,
 	})
@@ -436,15 +526,20 @@ type HTTPMethod string
 var (
 	// GET represents an HTTP "GET" method.
 	GET HTTPMethod = "GET"
-
+	// HEAD represents an HTTP "HEAD" method.
+	HEAD HTTPMethod = "HEAD"
+	// PUT represents an HTTP "PUT" method.
+	PUT HTTPMethod = "PUT"
 	// POST represents an HTTP "POST" method.
 	POST HTTPMethod = "POST"
+	// DELETE represents an HTTP "DELETE" method.
+	DELETE HTTPMethod = "DELETE"
 )
 
 // CreateTempURLOpts are options for creating a temporary URL for an object.
 type CreateTempURLOpts struct {
 	// (REQUIRED) Method is the HTTP method to allow for users of the temp URL.
-	// Valid values are "GET" and "POST".
+	// Valid values are "GET", "HEAD", "PUT", "POST" and "DELETE".
 	Method HTTPMethod
 
 	// (REQUIRED) TTL is the number of seconds the temp URL should be active.
@@ -454,56 +549,115 @@ type CreateTempURLOpts struct {
 	// the object path is used in the hash, the object URL needs to be parsed. If
 	// empty, the default OpenStack URL split point will be used ("/v1/").
 	Split string
+
+	// (Optional) Timestamp is the current timestamp used to calculate the Temp URL
+	// signature. If not specified, the current UNIX timestamp is used as the base
+	// timestamp.
+	Timestamp time.Time
+
+	// (Optional) TempURLKey overrides the Swift container or account Temp URL key.
+	// TempURLKey must correspond to a target container/account key, otherwise the
+	// generated link will be invalid. If not specified, the key is obtained from
+	// a Swift container or account.
+	TempURLKey string
+
+	// (Optional) Digest specifies the cryptographic hash function used to
+	// calculate the signature. Valid values include sha1, sha256, and
+	// sha512. If not specified, the default hash function is sha1.
+	Digest string
 }
 
 // CreateTempURL is a function for creating a temporary URL for an object. It
 // allows users to have "GET" or "POST" access to a particular tenant's object
 // for a limited amount of time.
 func CreateTempURL(c *gophercloud.ServiceClient, containerName, objectName string, opts CreateTempURLOpts) (string, error) {
-	if opts.Split == "" {
-		opts.Split = "/v1/"
-	}
-	duration := time.Duration(opts.TTL) * time.Second
-	expiry := time.Now().Add(duration).Unix()
-	getHeader, err := containers.Get(c, url.QueryEscape(containerName), nil).Extract()
+	url, err := getURL(c, containerName, objectName)
 	if err != nil {
 		return "", err
 	}
-	tempURLKey := getHeader.TempURLKey
+
+	if opts.Split == "" {
+		opts.Split = "/v1/"
+	}
+
+	// Initialize time if it was not passed as opts
+	date := opts.Timestamp
+	if date.IsZero() {
+		date = time.Now()
+	}
+	duration := time.Duration(opts.TTL) * time.Second
+	// UNIX time is always UTC
+	expiry := date.Add(duration).Unix()
+
+	// Initialize the tempURLKey to calculate a signature
+	tempURLKey := opts.TempURLKey
 	if tempURLKey == "" {
-		// fallback to an account TempURL key
-		getHeader, err := accounts.Get(c, nil).Extract()
+		// fallback to a container TempURL key
+		getHeader, err := containers.Get(c, containerName, nil).Extract()
 		if err != nil {
 			return "", err
 		}
 		tempURLKey = getHeader.TempURLKey
+		if tempURLKey == "" {
+			// fallback to an account TempURL key
+			getHeader, err := accounts.Get(c, nil).Extract()
+			if err != nil {
+				return "", err
+			}
+			tempURLKey = getHeader.TempURLKey
+		}
+		if tempURLKey == "" {
+			return "", ErrTempURLKeyNotFound{}
+		}
 	}
+
 	secretKey := []byte(tempURLKey)
-	url := getURL(c, url.QueryEscape(containerName), url.QueryEscape(objectName))
 	splitPath := strings.Split(url, opts.Split)
 	baseURL, objectPath := splitPath[0], splitPath[1]
 	objectPath = opts.Split + objectPath
 	body := fmt.Sprintf("%s\n%d\n%s", opts.Method, expiry, objectPath)
-	hash := hmac.New(sha1.New, secretKey)
+	var hash hash.Hash
+	switch opts.Digest {
+	case "", "sha1":
+		hash = hmac.New(sha1.New, secretKey)
+	case "sha256":
+		hash = hmac.New(sha256.New, secretKey)
+	case "sha512":
+		hash = hmac.New(sha512.New, secretKey)
+	default:
+		return "", ErrTempURLDigestNotValid{Digest: opts.Digest}
+	}
 	hash.Write([]byte(body))
 	hexsum := fmt.Sprintf("%x", hash.Sum(nil))
 	return fmt.Sprintf("%s%s?temp_url_sig=%s&temp_url_expires=%d", baseURL, objectPath, hexsum, expiry), nil
 }
 
 // BulkDelete is a function that bulk deletes objects.
+// In Swift, the maximum number of deletes per request is set by default to 10000.
+//
+// See:
+// * https://github.com/openstack/swift/blob/6d3d4197151f44bf28b51257c1a4c5d33411dcae/etc/proxy-server.conf-sample#L1029-L1034
+// * https://github.com/openstack/swift/blob/e8cecf7fcc1630ee83b08f9a73e1e59c07f8d372/swift/common/middleware/bulk.py#L309
 func BulkDelete(c *gophercloud.ServiceClient, container string, objects []string) (r BulkDeleteResult) {
-	// urlencode object names to be on the safe side
-	// https://github.com/openstack/swift/blob/stable/train/swift/common/middleware/bulk.py#L160
-	// https://github.com/openstack/swift/blob/stable/train/swift/common/swob.py#L302
-	encodedObjects := make([]string, len(objects))
-	for i, v := range objects {
-		encodedObjects[i] = strings.Join([]string{
-			url.QueryEscape(container),
-			url.QueryEscape(v)},
-			"/")
+	err := containers.CheckContainerName(container)
+	if err != nil {
+		r.Err = err
+		return
 	}
-	b := strings.NewReader(strings.Join(encodedObjects, "\n") + "\n")
-	resp, err := c.Post(bulkDeleteURL(c), b, &r.Body, &gophercloud.RequestOpts{
+
+	var body bytes.Buffer
+	for i := range objects {
+		if objects[i] == "" {
+			r.Err = fmt.Errorf("object names must not be the empty string")
+			return
+		}
+		body.WriteString(container)
+		body.WriteRune('/')
+		body.WriteString(objects[i])
+		body.WriteRune('\n')
+	}
+
+	resp, err := c.Post(bulkDeleteURL(c), &body, &r.Body, &gophercloud.RequestOpts{
 		MoreHeaders: map[string]string{
 			"Accept":       "application/json",
 			"Content-Type": "text/plain",
