@@ -53,6 +53,7 @@ var createResponse = `{
 			"replication_type": null,
 			"task_state": null,
 			"snapshot_support": true,
+			"create_share_from_snapshot_support": true,
 			"consistency_group_id": "9397c191-8427-4661-a2e8-b23820dc01d4",
 			"source_cgsnapshot_member_id": null,
 			"volume_type": "default",
@@ -133,6 +134,7 @@ var updateResponse = `
 		"task_state": null,
 		"is_public": false,
 		"snapshot_support": true,
+		"create_share_from_snapshot_support": true,
 		"name": "my_new_test_share",
 		"created_at": "2015-09-18T10:25:24.000000",
 		"share_proto": "NFS",
@@ -188,6 +190,7 @@ var getResponse = `{
         "task_state": null,
         "is_public": true,
         "snapshot_support": true,
+        "create_share_from_snapshot_support": true,
         "name": "my_test_share",
         "created_at": "2015-09-18T10:25:24.000000",
         "share_proto": "NFS",
@@ -241,6 +244,7 @@ var listDetailResponse = `{
 		        "task_state": null,
 		        "is_public": true,
 		        "snapshot_support": true,
+		        "create_share_from_snapshot_support": true,
 		        "name": "my_test_share",
 		        "created_at": "2015-09-18T10:25:24.000000",
 		        "share_proto": "NFS",
@@ -528,12 +532,6 @@ func MockUpdateMetadataResponse(t *testing.T) {
 	})
 }
 
-var deleteMetadatumRequest = `{
-		"metadata": {
-			"foo": "bar"
-		}
-	}`
-
 // MockDeleteMetadatumResponse creates a mock unset metadata response
 func MockDeleteMetadatumResponse(t *testing.T, key string) {
 	th.Mux.HandleFunc(shareEndpoint+"/"+shareID+"/metadata/"+key, func(w http.ResponseWriter, r *http.Request) {
